import {
  createEndpointRuntime,
  createStreamWirings
} from "./chunk-PNUPTSM5.js";
import {
  parseEndpoint
} from "./chunk-QW7OLTWX.js";

// src/background.ts
import browser from "webextension-polyfill";
var messageQueue = /* @__PURE__ */ new Set();
var portMap = /* @__PURE__ */ new Map();
var endpointRuntime = createEndpointRuntime("background", (message) => {
  if (message.origin.context === "background" && message.destination.context !== "background" && !message.destination.tabId)
    throw new TypeError("When sending messages from background page, use @tabId syntax to target specific tab");
  const { context: destName, tabId: destTabId, frameId: destFrameId } = message.destination;
  const { tabId: srcTabId } = message.origin;
  message.destination.tabId = null;
  message.destination.frameId = null;
  let resolvedDestination = ["popup", "options"].includes(destName) ? destName : `${destName === "window" ? "content-script" : destName}@${destTabId || srcTabId}`;
  if (destFrameId)
    resolvedDestination = `${resolvedDestination}.${destFrameId}`;
  const destPort = portMap.get(resolvedDestination);
  if (destPort)
    destPort.postMessage(message);
  else
    messageQueue.add({ resolvedDestination, message });
});
browser.runtime.onConnect.addListener((incomingPort) => {
  let portId = incomingPort.name || `content-script@${incomingPort.sender.tab.id}`;
  const portFrame = incomingPort.sender.frameId;
  if (portFrame)
    portId = `${portId}.${portFrame}`;
  const { context, tabId: linkedTabId } = parseEndpoint(portId);
  if (!linkedTabId && context !== "popup" && context !== "options")
    return;
  portMap.set(portId, incomingPort);
  messageQueue.forEach((queuedMsg) => {
    if (queuedMsg.resolvedDestination === portId) {
      incomingPort.postMessage(queuedMsg.message);
      messageQueue.delete(queuedMsg);
    }
  });
  incomingPort.onDisconnect.addListener(() => {
    if (portMap.get(portId) === incomingPort)
      portMap.delete(portId);
  });
  incomingPort.onMessage.addListener((message) => {
    var _a;
    if ((_a = message == null ? void 0 : message.origin) == null ? void 0 : _a.context) {
      message.origin.tabId = linkedTabId;
      endpointRuntime.handleMessage(message);
    }
  });
});
var { sendMessage, onMessage } = endpointRuntime;
var { openStream, onOpenStreamChannel } = createStreamWirings(endpointRuntime);
export {
  onMessage,
  onOpenStreamChannel,
  openStream,
  sendMessage
};
