// src/internal/message-port.ts
var promise;
var portOfferEvent = "webext-port-offer";
var getMessagePort = (thisContext, namespace, onMessage) => promise != null ? promise : promise = new Promise((resolve) => {
  const acceptMessagingPort = ({ detail: { scope, context, port } }) => {
    if (scope === namespace && context !== thisContext) {
      console.log("using inbound port in", thisContext);
      window.removeEventListener(portOfferEvent, acceptMessagingPort);
      port.onmessage = onMessage;
      port.postMessage("port-accepted");
      return resolve(port);
    }
  };
  const offerMessagingPort = () => {
    const channel = new MessageChannel();
    channel.port1.onmessage = (event) => {
      if (event.data === "port-accepted") {
        console.log("using own port in", thisContext);
        window.removeEventListener(portOfferEvent, acceptMessagingPort);
        return resolve(channel.port1);
      }
      onMessage == null ? void 0 : onMessage(event);
    };
    window.dispatchEvent(new CustomEvent(portOfferEvent, {
      detail: {
        port: channel.port2,
        scope: namespace,
        context: thisContext
      }
    }));
  };
  window.addEventListener(portOfferEvent, acceptMessagingPort);
  if (thisContext === "window")
    setTimeout(offerMessagingPort, 0);
  else
    offerMessagingPort();
});

// src/internal/post-message.ts
var usePostMessaging = (thisContext) => {
  let allocatedNamespace;
  let messagingEnabled = false;
  let onMessageCallback;
  let portP;
  return {
    enable: () => messagingEnabled = true,
    onMessage: (cb) => onMessageCallback = cb,
    postMessage: async (msg) => {
      if (thisContext !== "content-script" && thisContext !== "window")
        throw new Error("Endpoint does not use postMessage");
      if (!messagingEnabled)
        throw new Error("Communication with window has not been allowed");
      ensureNamespaceSet(allocatedNamespace);
      return (await portP).postMessage(msg);
    },
    setNamespace: (nsps) => {
      if (allocatedNamespace)
        throw new Error("Namespace once set cannot be changed");
      allocatedNamespace = nsps;
      portP = getMessagePort(thisContext, nsps, ({ data }) => onMessageCallback == null ? void 0 : onMessageCallback(data));
    }
  };
};
function ensureNamespaceSet(namespace) {
  if (typeof namespace !== "string" || namespace.length === 0) {
    throw new Error(`webext-bridge uses window.postMessage to talk with other "window"(s), for message routing and stuff,which is global/conflicting operation in case there are other scripts using webext-bridge. Call Bridge#setNamespace(nsps) to isolate your app. Example: setNamespace('com.facebook.react-devtools'). Make sure to use same namespace across all your scripts whereever window.postMessage is likely to be used\``);
  }
}

export {
  usePostMessaging
};
